@@ -7,8 +7,8 @@ __all__ = [
 
 __package_name__ = 'pyrcs'
 __package_name_alt__ = 'PyRCS'
-__version__ = '0.2.14'
+__version__ = '0.2.15'
 __author__ = 'Qian Fu'
 __email__ = 'qian.fu@outlook.com'
-__description__ = \
-    'An open-source tool for collecting railway codes used in different UK rail industry systems.'
+__description__ = 'PyRCS: an open-source tool for collecting railway codes ' \
+                  'used in different UK rail industry systems.'
